import urllib
import json
import os
import requests
import webbrowser
from oauthlib.oauth2 import WebApplicationClient
import pathlib

DEFAULT_CONFIG = {"hangthreshold": 120, "link": {}}
SCRIPT_LOCATION = pathlib.Path(os.path.realpath(os.path.dirname(__file__)))
CONFIG_LOCATION = SCRIPT_LOCATION / 'config.json'


def main():
    if not os.path.exists(CONFIG_LOCATION):
        with open(CONFIG_LOCATION, 'w') as f:
            json.dump(DEFAULT_CONFIG, f, ensure_ascii=False, indent=4,
                      sort_keys=True, separators=(',', ': '))

    CLIENT_ID = "4837413857745.4837415034865"
    REQUEST_SCOPE = "incoming-webhook"
    CLIENT_SECRET = "94bd2a56603cfc13ab47dc851064f2eb"
    REDIRECT_URL = "https://sleepy-shelf-95701.herokuapp.com/slack"

    oauth = WebApplicationClient(CLIENT_ID)
    url, headers, body = oauth.prepare_authorization_request('https://slack.com/oauth/authorize', scope=REQUEST_SCOPE, redirect_url=REDIRECT_URL)

    webbrowser.open(url)
    print("Please follow the authentication process in your browser.")
    CODE = input("Please paste your code here: ")

    request_data = {
        "client_id": CLIENT_ID,
        "code": CODE,
        "client_secret": CLIENT_SECRET,
        "scope": REQUEST_SCOPE,
        "redirect_url": REDIRECT_URL
    }

    url, headers, body = oauth.prepare_token_request('https://slack.com/api/oauth.access', code=CODE, client_secret=CLIENT_SECRET)
    req = urllib.request.Request(url, body.encode(), headers=headers)
    payload = json.loads(urllib.request.urlopen(req).read().decode('utf-8'))

    if payload["ok"]:
        team_name = payload["team_name"],
        channel = payload["incoming_webhook"]["channel"],
        url = payload["incoming_webhook"]["url"]

        with open(CONFIG_LOCATION, 'r') as f:
            config = json.load(f)
            config["link"] = {
                "team_name": team_name,
                "channel": channel,
                "url": url
            }

        with open(CONFIG_LOCATION, 'w') as f:
            json.dump(config, f, ensure_ascii=False, indent=4,
                      sort_keys=True, separators=(',', ': '))

        print(f"Now set to {channel[0]} at {team_name[0]}!")

    else:
        errormsg = payload["error"]
        print(f"An error occured during authentication. Please try again. ({errormsg})")


if __name__ == "__main__":
    main()


import requests
import datetime
import time


def slack_notify(
    link,
    type,
    cmd,
    starttime,
    lastline,
    endtime=0,
    errmsg="",
    returncode=0,
    regex="",
    hangth=2
):
    blocks = []
    cmdline = " ".join(cmd)
    if lastline == "" or lastline == "\n":
        lastline = "No output yet"
    else:
        lastline = f"```{lastline}```"

    if errmsg == "" or errmsg == "\n":
        errmsg = "No error message"
    else:
        errmsg = f"```{errmsg}```"

    if type == "regex_notify":
        notificationline = "🔔 Regex match"
        blocks = [
            {
                "type": "divider"
            },
            {
                "type": "header",
                "text": {
                    "type": "plain_text",
                    "text": ":bell: Regex match"
                }
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": f"Detected regex match for `{regex}`."
                }
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": f"```\n $ {cmdline}```",
                }
            },
            {
                "type": "section",
                "fields": [
                    {
                        "type": "mrkdwn",
                        "text": "*Started on:*\n<!date^" + str(int(starttime))
                                + "^ {date_num} {time_secs}|Date and time not available>\n"
                    },
                    {
                        "type": "mrkdwn",
                        "text": f"*Duration:*\n {datetime.timedelta(seconds=int(time.time() - starttime))}\n"
                    }
                ]
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": f"*Last line:*\n {lastline}",
                }
            }
        ]

    if type == "successful":
        notificationline = "✅ Process successfully completed"
        blocks = [
            {
                "type": "divider"
            },
            {
                "type": "header",
                "text": {
                    "type": "plain_text",
                    "text": ":white_check_mark: Process successfully completed"
                }
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": f"```\n $ {cmdline}```",
                }
            },
            {
                "type": "section",
                "fields": [
                    {
                        "type": "mrkdwn",
                        "text": f"*Return code:*\n `{returncode}`\n"
                    },
                    {
                        "type": "mrkdwn",
                        "text": f"*Duration:*\n {datetime.timedelta(seconds=int(endtime - starttime))}\n"
                    },
                    {
                        "type": "mrkdwn",
                        "text": "*Started on:*\n<!date^" + str(int(starttime))
                                + "^ {date_num} {time_secs}|Date and time not available>\n"
                    },
                    {
                        "type": "mrkdwn",
                        "text": "*Ended on:*\n<!date^" + str(int(endtime))
                                + "^ {date_num} {time_secs}|Date and time not available>\n"
                    }
                ]
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": f"*Last line:*\n {lastline}",
                }
            }
        ]

    if type == "timeout":
        notificationline = "⚠️ Process may be hanging"
        blocks = [
            {
                "type": "divider"
            },
            {
                "type": "header",
                "text": {
                    "type": "plain_text",
                    "text": ":warning: Process may be hanging"
                }
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": f"Process has not been outputting for {hangth} seconds.\nYou can change this threshold with `--hangthreshold` argument."
                }
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": f"```\n $ {cmdline}```",
                }
            },
            {
                "type": "section",
                "fields": [
                    {
                        "type": "mrkdwn",
                        "text": "*Started on:*\n<!date^" + str(int(starttime))
                                + "^ {date_num} {time_secs}|Date and time not available>\n"
                    },
                    {
                        "type": "mrkdwn",
                        "text": f"*Duration:*\n {datetime.timedelta(seconds=int(time.time() - starttime))}\n"
                    }
                ]
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": f"*Last line:*\n {lastline}",
                }
            }
        ]

    if type == "ended with error":
        notificationline = "❌ Process ended with error"
        blocks = [
            {
                "type": "divider"
            },
            {
                "type": "header",
                "text": {
                    "type": "plain_text",
                    "text": ":x: Process ended with error"
                }
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": f"```\n $ {cmdline}```",
                }
            },
            {
                "type": "section",
                "fields": [
                    {
                        "type": "mrkdwn",
                        "text": f"*Return code:*\n `{returncode}`\n"
                    },
                    {
                        "type": "mrkdwn",
                        "text": f"*Duration:*\n {datetime.timedelta(seconds=int(endtime - starttime))}\n"
                    },
                    {
                        "type": "mrkdwn",
                        "text": "*Started on:*\n<!date^" + str(int(starttime))
                                + "^ {date_num} {time_secs}|Date and time not available>\n"
                    },
                    {
                        "type": "mrkdwn",
                        "text": "*Ended on:*\n<!date^" + str(int(endtime))
                                + "^ {date_num} {time_secs}|Date and time not available>\n"
                    }
                ]
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": f"*Last line:*\n {lastline}",
                }
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": f"*Error:*\n {errmsg}",
                }
            }
        ]

    requests.post(link,
                  json={
                      "title": "Lookout notification",
                      "blocks": blocks,
                      "text": notificationline,
                  }
                  )



# import os
import sys
import subprocess
from . import notify
# import notify
import time
import signal
import os
import json
import argparse
from . import auth
# import auth
import pathlib
import re

SCRIPT_LOCATION = pathlib.Path(os.path.realpath(os.path.dirname(__file__)))
CONFIG_LOCATION = SCRIPT_LOCATION / 'config.json'


class timeout:
    def __init__(self, link, cmd, starttime, lastline, seconds=1):
        self.seconds = seconds
        self.cmd = cmd
        self.lastline = lastline
        self.starttime = starttime
        self.link = link

    def handle_timeout(self, signum, frame):
        notify.slack_notify(
            self.link,
            "timeout",
            cmd=self.cmd,
            starttime=self.starttime,
            lastline=self.lastline,
            hangth=self.seconds
        )

    def __enter__(self):
        signal.signal(signal.SIGALRM, self.handle_timeout)
        signal.alarm(self.seconds)

    def __exit__(self, type, value, traceback):
        signal.alarm(0)


def getchar(p):
    c = p.stdout.read(1).decode(sys.stdout.encoding)
    if c == "":
        return -1
    else:
        return c


def getcharerr(p):
    c = p.stderr.read(1).decode(sys.stdout.encoding)
    if c == "":
        return -1
    else:
        return c


def main():
    # parse
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--hangthreshold',
        '-ht',
        help='Override config on hang threshold for sending hanging alert [seconds]',
        type=int,
        default=-1
    )
    parser.add_argument(
        '--regex',
        help='Send alert when output matches regex',
        type=str,
        default="$."
    ),
    parser.add_argument(
        '--change',
        help='Change alert channel.',
        action='store_true',
        default=False
    ),
    parser.add_argument(
        '--reset',
        help='Revert to factory settings',
        action='store_true',
        default=False
    )
    parser.add_argument(
        "command",
        help='Command to be executed.',
        nargs=argparse.REMAINDER
    )
    args = parser.parse_args()

    # if reset is True
    if args.reset:
        os.remove(CONFIG_LOCATION)
        auth.main()

    # load configuration
    if os.path.exists(CONFIG_LOCATION):
        with open(CONFIG_LOCATION, 'r') as f:
            config = json.load(f)

        if len(config["link"]) == 0:
            auth.main()
    else:
        auth.main()
        with open(CONFIG_LOCATION, 'r') as f:
            config = json.load(f)

    # if hangthreshold is defined
    try:
        if args.hangthreshold != -1:
            config["hangthreshold"] = args.hangthreshold
            with open(CONFIG_LOCATION, 'w') as f:
                json.dump(config, f, ensure_ascii=False, indent=4,
                          sort_keys=True, separators=(',', ': '))
    except KeyError:
        raise ValueError("Could not read config.json file. Consider using --reset to reset.")

    # load link
    try:
        link = config["link"]["url"]
        HANGTHRESHOLD = config["hangthreshold"]

    except KeyError:
        raise ValueError("Could not read config.json file. Consider using --reset to reset.")

    # regex
    # pattern = re.compile(args.regex)

    # if change is True
    if args.change:
        auth.main()

    if len(args.command) == 0:
        parser.print_help(sys.stderr)
        sys.exit(0)

    # start ------------------------
    starttime = time.time()
    line = ""
    lastline = ""
    stderr_record = ""
    line_reported = False

    p = subprocess.Popen(args.command, bufsize=0, stdin=sys.stdin,
                         stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    # for each output char
    while p.poll() is None:
        nomore = False
        while not nomore:
            # while p.stdout.:
            with timeout(
                link=link,
                cmd=args.command,
                starttime=starttime,
                lastline=line,
                seconds=HANGTHRESHOLD
            ):
                c = getchar(p)
            if c == '\n':
                lastline = line
                line = ""
                line_reported = False
            if c != -1:
                line += c
                sys.stdout.write(c)
                sys.stdout.flush()
            else:
                nomore = True
                continue
            if re.search(args.regex, line) and not line_reported:
                notify.slack_notify(
                    link,
                    "regex_notify",
                    cmd=args.command,
                    lastline=line,
                    starttime=starttime,
                    regex=args.regex
                )
                line_reported = True

        nomore = False
        while not nomore:
            c = getcharerr(p)
            if c == -1:
                nomore = True
                continue
            else:
                nomore = False
            if c != -1:
                sys.stderr.write(c)
                stderr_record += c
                sys.stderr.flush()

    # process ended
    endtime = time.time()
    if p.returncode == 0:
        notify.slack_notify(
            link,
            "successful",
            cmd=args.command,
            lastline=lastline,
            starttime=starttime,
            endtime=endtime,
            returncode=p.returncode
        )
    else:
        notify.slack_notify(
            link,
            "ended with error",
            cmd=args.command,
            lastline=lastline,
            errmsg=stderr_record,
            starttime=starttime,
            endtime=endtime,
            returncode=p.returncode
        )


if __name__ == "__main__":
    main()
